unit xmmutil;

interface

uses
  Windows;

function GetAlignMem(Size, Align: DWORD): Pointer;
procedure FreeAlignMem(p: Pointer);
function CheckCPU: DWORD;

const
  SIMD_MMX      = $01;
  SIMD_SSE      = $02;
  SIMD_SSE2     = $04;
  SIMD_MMX_EX   = $08;
  SIMD_3DNOW    = $10;
  SIMD_3DNOW_EX = $20;
  
implementation

function CheckCPU: DWORD;
var
  Flags: DWORD;
begin
  asm
    push  ebx
    push  edi

    mov   eax, 1
    xor   edi, edi
    cpuid
    test  edx, 1 shl 23
    jz    @check_sse
    or    edi, SIMD_MMX

  @check_sse:
    test  edx, 1 shl 25
    jz    @check_sse2
    or    edi, SIMD_SSE

  @check_sse2:
    test  edx, 1 shl 26
    jz    @check_mmx_ex
    or    edi, SIMD_SSE2

  @check_mmx_ex:
    mov   eax, $80000000
    cpuid
    cmp   eax, $80000001
    jb    @check_end
    mov   eax, $80000001
    cpuid
    test  edx, 1 shl 22
    jz    @check_3dnow
    or    edi, SIMD_MMX_EX

  @check_3dnow:
    test  edx, 1 shl 31
    jz    @check_3dnow_ex
    or    edi, SIMD_3DNOW

  @check_3dnow_ex:
    test  edx, 1 shl 30
    jz    @check_end
    or    edi, SIMD_3DNOW_EX

  @check_end:
    mov   Flags, edi
    pop   edi
    pop   ebx
  end;
  if Flags and SIMD_SSE <> 0 then
  begin
    try
      asm
        movaps  xmm0, xmm0
      end;
    except
      Flags := Flags and not SIMD_SSE;
    end;
  end;
  if Flags and SIMD_SSE2 <> 0 then
  begin
    try
      asm
        movdqa  xmm0, xmm0
      end;
    except
      Flags := Flags and not SIMD_SSE2;
    end;
  end;
  Result := Flags;
end;

function GetAlignMem(Size, Align: DWORD): Pointer;
var
  p: Pointer;
begin
  if Align < 4 then Align := 4;
  Dec(Align);
  GetMem(p, Size + Align + 4);
  Result := Pointer((DWORD(p)+Align+4) and not Align);
  PPointer(DWORD(Result)-4)^ := p;
end;

procedure FreeAlignMem(p: Pointer);
var
  ptr: Pointer;
begin
  ptr := PPointer(DWORD(p)-4)^;
  FreeMem(ptr);
end;

end.
